import os

from analogai.foundation.common.utils.env_utils import (
	load_envs_in_order,
	parse_bool,
	parse_optional_float,
)


load_envs_in_order()

USE_VECTOR_SEARCH = parse_bool(os.getenv("USE_VECTOR_SEARCH"), True)
SIMILARITY_CUTOFF = parse_optional_float(os.getenv("SIMILARITY_CUTOFF"), 0.4)

MONGODB_URI = os.getenv("MONGODB_URI")
MONGODB_DATABASE = os.getenv("MONGODB_DATABASE")

NEO4J_URI = os.getenv("NEO4J_URI")
NEO4J_USER = os.getenv("NEO4J_USER")
NEO4J_PASSWORD = os.getenv("NEO4J_PASSWORD")
NEO4J_DATABASE = os.getenv("NEO4J_DATABASE")
NEO4J_IS_ENCRYPTED = os.getenv("NEO4J_IS_ENCRYPTED", "false")
